#pragma once

#include "MeshDatatypes.h"
#include "Predicates.h"
#include <string>
#include <vector>
#include <map>
#include <set>
#include <iterator>
#include <unordered_map>
#include <unordered_set>

class MeshLoader {
public:
	MeshLoader();
	MeshLoader(const std::string& p_fileName);
	~MeshLoader();
		
	virtual void load() = 0;

	void printNodes();
	void printFiniteElems();
	void printBoundaryFiniteElements();

	const std::vector<Node>& getNodes();
	const std::unordered_map<uint16_t, FiniteElement>& getFiniteElems();
	const std::unordered_map<uint16_t, BoundaryFiniteElement>& getBoundaryFiniteElements();

	std::vector<FiniteElement> getFEbyNodesIds(uint16_t id1, uint16_t id2, uint16_t id3);
	std::vector<std::unordered_map<uint16_t, FiniteElement>::iterator> getFEbyEdge(uint16_t id1, uint16_t id2);

	std::vector<BoundaryFiniteElement> getBFEbySurfaceID(uint16_t surID);
	std::vector<std::unordered_map<uint16_t, BoundaryFiniteElement>::iterator> getBFEbyEdge(uint16_t id1, uint16_t id2);

	std::vector<Node> getNodesbySurfaceID(uint16_t surID);
	std::vector<FiniteElement> getFEbyMaterialID(uint16_t areaID);

	std::vector<std::set<uint16_t>> nodeNeighbours();

	bool insertMiddlesOfEdges();


protected:
	std::string fileName;

	std::vector<Node> nodes;                      //ID  -    
	std::unordered_map<uint16_t, FiniteElement> finiteElements;                  //ID  -  
	std::unordered_map<uint16_t, BoundaryFiniteElement> boundaryFiniteElements;    //  - ID
	std::unordered_set<std::pair<uint16_t, uint16_t>, edge_pair_hash> alreadyHasMiddle;

	std::unordered_set<std::pair<uint16_t, uint16_t>, edge_pair_hash> boundaryEdges;

	unsigned nodesCount;
};